//
//  GameScene.swift
//  DouShouQi_App
//
//  Created by Nathan Verdier on 28/05/2024.
//

import Foundation
import SpriteKit
import DouShouQiModel

class GameScene : SKScene {
    
    var player1: HumanPlayer = HumanPlayer(withName: "Rémi", andId: .player1)!
    var player2: HumanPlayer = HumanPlayer(withName: "Rayhan", andId: .player2)!
    
    var game: Game
    
    let pieces: [Owner : [Animal:SpriteMeeple]] = [
        .player1: [
            .rat: SpriteMeeple(imageNamed: AppImages.Rat, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .cat: SpriteMeeple(imageNamed: AppImages.Cat, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .dog: SpriteMeeple(imageNamed: AppImages.Dog, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .wolf: SpriteMeeple(imageNamed: AppImages.Wolf, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .leopard: SpriteMeeple(imageNamed: AppImages.Leopard, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .lion: SpriteMeeple(imageNamed: AppImages.Lion, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .tiger: SpriteMeeple(imageNamed: AppImages.Tigger, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .elephant: SpriteMeeple(imageNamed: AppImages.Elephant, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
        ],
        .player2: [
            .rat: SpriteMeeple(imageNamed: AppImages.Rat, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .cat: SpriteMeeple(imageNamed: AppImages.Cat, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .dog: SpriteMeeple(imageNamed: AppImages.Dog, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .wolf: SpriteMeeple(imageNamed: AppImages.Wolf, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .leopard: SpriteMeeple(imageNamed: AppImages.Leopard, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .lion: SpriteMeeple(imageNamed: AppImages.Lion, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .tiger: SpriteMeeple(imageNamed: AppImages.Tigger, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .elephant: SpriteMeeple(imageNamed: AppImages.Elephant, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
        ],
    ]
    
    let imageBoard: SKSpriteNode = SKSpriteNode(imageNamed: AppImages.boardGame)
    
    override init(size: CGSize) {
        game = try! Game(withRules: ClassicRules(), andPlayer1: player1, andPlayer2: player2)
        super.init(size: size)
        imageBoard.size = size
        //self.scaleMode = .aspectFit
        self.anchorPoint = CGPoint(x: 0.5, y: 0.5)
        self.backgroundColor = .white
        
        self.addChild(imageBoard)
        
        for piece in pieces.flatMap({owner, pieces in pieces.values}) {
            self.addChild(piece)
            piece.setOnMove(onMove: onMeepleMove)
        }
        
        initializeBoard(game.board)
        
        game.addInvalidMoveCallbacksListener { _, move, player, result in
            if result {
                return
            }
            print("**************************************")
            print("⚠️⚠️⚠️⚠️ Invalid Move detected: \(move) by \(player.name) (\(player.id))")
            print("**************************************")
           
        }
        
        Task {
            try await game.start()
        }
    }
    
    func initializeBoard(_ board: Board) {
        for (lineIndex, currentLine) in game.board.grid.enumerated() {
            for (cellIndex, currentCell) in currentLine.enumerated() {
                if let piece = currentCell.piece {
                    pieces[piece.owner]?[piece.animal]?.cellPosition = CGPoint(x: cellIndex-3, y: lineIndex-4)
                    pieces[piece.owner]?[piece.animal]?.cellPosition = CGPoint(x: 0, y: 0)
                    print("line :", lineIndex, " cologne:  ", cellIndex)
                }
            }
        }
    }
    
    required init?(coder aDecoder: NSCoder) {
        game = try! Game(withRules: ClassicRules(), andPlayer1: player1, andPlayer2: player2)
        super.init(coder: aDecoder);
    }
    
    func onMeepleMove(_ start: CGPoint, _ end: CGPoint) {
        
        let owner = game.rules.getNextPlayer()
        let player = game.players[owner]
        
        let move = Move(of: owner, fromRow: Int(start.x), andFromColumn: Int(start.y), toRow: Int(end.x), andToColumn: Int(end.y))
        
        print("Meeple moved = ", start, " -> ", end)
        
        Task {
            try! await (player as! HumanPlayer).chooseMove(move)
        }
    }
    
    public func start() async throws {
        try await game.start()
    }
}
